package com.fr.plugin.xx.wechat.img.output.handle;

import com.fr.base.Formula;
import com.fr.io.TemplateWorkBookIO;
import com.fr.io.exporter.ImageExporter;
import com.fr.json.JSONObject;
import com.fr.main.TemplateWorkBook;
import com.fr.main.workbook.ResultWorkBook;
import com.fr.plugin.xx.wechat.img.output.LogUtils;
import com.fr.plugin.xx.wechat.img.output.conf.ImgPushConfig;
import com.fr.plugin.xx.wechat.img.output.entity.ImgConfigEntity;
import com.fr.plugin.xx.wechat.img.output.entity.OutputWebHook;
import com.fr.plugin.xx.wechat.img.output.service.ImgConfigService;
import com.fr.plugin.xx.wechat.img.output.util.HttpsUtil;
import com.fr.schedule.base.constant.ScheduleConstants;
import com.fr.schedule.feature.output.OutputActionHandler;
import com.fr.stable.StringUtils;
import com.fr.stable.ViewActor;
import com.fr.third.org.apache.commons.lang3.math.NumberUtils;
import com.fr.third.springframework.core.io.ByteArrayResource;
import com.fr.third.springframework.http.HttpEntity;
import com.fr.third.springframework.http.HttpHeaders;
import com.fr.third.springframework.http.MediaType;
import com.fr.third.springframework.http.ResponseEntity;
import com.fr.third.springframework.util.LinkedMultiValueMap;
import com.fr.third.springframework.util.MultiValueMap;
import com.fr.third.springframework.web.client.RestTemplate;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @Author xx
 * @Date 2020/9/15
 * @Description
 **/
public class WebHookHandle extends OutputActionHandler<OutputWebHook> {

    private final static String UPLOAD = "%s/wx-work/open-api/media/upload";

    private final static String MESSAGE = "%s/wx-work/open-api/insideMessage/send";

    private final static String TOKEN = "%s/wx-work/open-api/login?apiUserCode=%s&apiUserSecret=%s";

    @Override
    public void doAction(OutputWebHook outputWebHook, Map<String, Object> map) throws Exception {
        Integer pool = 10;
        String upname = getParam(map, "upname");
        if(StringUtils.isBlank(upname)){
            throw new Exception("upname参数不能为空");
        }
        String cptName = getParam(map, "cpt");
        if(StringUtils.isBlank(cptName)){
            throw new Exception("cpt参数不能为空");
        }
        String poolSize = getParam(map, "poolSize");
        if (NumberUtils.isCreatable(poolSize)) {
            pool = Integer.valueOf(poolSize);
        }
        String date = getParam(map, "date");
        List<ImgConfigEntity> list = ImgConfigService.getListByUpName(upname);
        ExecutorService es = Executors.newFixedThreadPool(pool);
        String token = getToken();
        if (StringUtils.isBlank(token)) {
            throw new Exception("token is null");
        }
        Map<String, Object> header = new HashMap<>();
        header.put("X-Auth-Token", token);
        for (ImgConfigEntity e : list) {
            es.submit(() -> {
                ByteArrayOutputStream os = new ByteArrayOutputStream();
                try {
                    Map<String, Object> params = e.toMap();
                    if(StringUtils.isNotBlank(date)){
                        params.put("date",date);
                    }
                    TemplateWorkBook wb = TemplateWorkBookIO.readTemplateWorkBook(cptName);
                    ImageExporter exporter = new ImageExporter("png", 96);
                    ResultWorkBook resultWorkBook = wb.execute(params, new ViewActor());
                    exporter.export(os, resultWorkBook);
                    LogUtils.debug4plugin("execute temp {} by param {}", cptName, params);
                    String mediaId = upload(os, token);
                    if (StringUtils.isBlank(mediaId)) {
                        LogUtils.warn("{} upload file error", e.getId());
                        return;
                    }
                    sendMess(e.getActNumber(), mediaId, header);
                } catch (Exception ex) {
                    LogUtils.error(ex.getMessage(), e);
                } finally {
                    try {
                        os.close();
                    } catch (IOException exception) {
                        LogUtils.error(exception.getMessage(), exception);
                    }
                }
            });
        }
    }

    private String getParam(Map<String, Object> map, String p) {
        List<Map> list = (List) map.get(ScheduleConstants.RECORD_LIST);
        Object title = StringUtils.EMPTY;
        if (list == null || list.isEmpty()) {
            return StringUtils.EMPTY;
        }
        Object param = list.get(0).get(p);
        if (param == null) {
            return StringUtils.EMPTY;
        }
        if (param instanceof Formula) {
            Formula formula = (Formula) param;
            title = formula.getResult();
        } else {
            title = param.toString();
        }
        return title.toString();
    }

    private String getToken() {
        ImgPushConfig config = ImgPushConfig.getInstance();
        String url = String.format(TOKEN, config.getHost(), config.getApiUserCode(), config.getApiUserSecret());
        LogUtils.debug4plugin("token url is {}", url);
        String res = HttpsUtil.sendPost(url, new HashMap(), JSONObject.create());
        LogUtils.debug4plugin("get token res is {}", res);
        JSONObject object = new JSONObject(res);
        return object.getJSONObject("data").getJSONObject("details").getString("token");
    }

    private void sendMess(String phone, String mediaId, Map<String, Object> header) {
        JSONObject image = JSONObject.create().put("mediaId", mediaId);
        JSONObject body = JSONObject.create().put("toPhone", phone).put("image", image).put("msgType", "image");
        LogUtils.debug4plugin("send to {} body is {}", phone, body.toString());
        String res = HttpsUtil.sendPost(String.format(MESSAGE, ImgPushConfig.getInstance().getHost()), header, body);
        LogUtils.debug4plugin("send to {} res is {}", phone, res);
    }

    private String upload(ByteArrayOutputStream os, String token) throws UnsupportedEncodingException {
        ByteArrayResource fileAsResource = new ByteArrayResource(os.toByteArray()) {
            @Override
            public String getFilename() {
                return UUID.randomUUID().toString() + ".png";
            }

            @Override
            public long contentLength() {
                return os.size();
            }
        };
        MultiValueMap<String, Object> multipartRequest = new LinkedMultiValueMap<>();
        multipartRequest.add("file", fileAsResource);
        multipartRequest.add("mediaType", "image");
        HttpHeaders headers = new HttpHeaders();
        headers.add("X-Auth-Token", token);
        headers.setContentType(MediaType.MULTIPART_FORM_DATA);
        HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity(multipartRequest, headers);
        ResponseEntity<String> res =
                new RestTemplate().postForEntity(String.format(UPLOAD, ImgPushConfig.getInstance().getHost()), requestEntity, String.class);
        //String upload = HttpsUtil.doEntityPost(String.format(UPLOAD, ImgPushConfig.getInstance().getHost()), header, builder.build());
        LogUtils.debug4plugin("send png res is {}", res.getBody());
        JSONObject object = new JSONObject(res.getBody());
        return object.getJSONObject("data").getString("mediaId");
    }

}
